#include "mram.h"

HAL_StatusTypeDef MRAM_Transaction(unsigned char ucCmd, unsigned char *Buffer, unsigned char Bytes);

void MRAM_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
#ifdef USE_MRAM_COMMAND
#ifdef USE_CMDLINE
	CmdLine_RegisterCmd("rdsr", "", MRAM_DBG, 1);	/* read status register */
	CmdLine_RegisterCmd("mtr", "xxx", MRAM_DBG, 0); /* Transaction command */
	CmdLine_RegisterCmd("mrb", "x", MRAM_DBG, 2); /* Read byte command */
	CmdLine_RegisterCmd("mwb", "xx", MRAM_DBG, 3); /* Write byte command */
#endif
#endif
	MRAM_Transaction(MR25H256_WREN, NULL, 0);
}

HAL_StatusTypeDef MRAM_Transaction(unsigned char ucCmd, unsigned char *Buffer, unsigned char Bytes)
{
	HAL_StatusTypeDef res;

	BSP_MRAM_CS_ACTIVE();
	res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, &ucCmd, 1, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return res;
	switch(ucCmd)
	{
	case MR25H256_WREN:
	case MR25H256_WRDI:
	case MR25H256_SLEEP:
	case MR25H256_WAKE:
		break;
	case MR25H256_RDSR:
		res = HAL_SPI_Receive(&MRAM_SPI_HANDLER, Buffer, 1, MRAM_SPI_TIMEOUT);
		if (res != HAL_OK) return res;
		break;
	case MR25H256_WRSR:
		res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, Buffer, 1, MRAM_SPI_TIMEOUT);
		if (res != HAL_OK) return res;
		break;
	case MR25H256_READ:
		res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, &Buffer[0], 2, MRAM_SPI_TIMEOUT);
		if (res != HAL_OK) return res;
		res = HAL_SPI_Receive(&MRAM_SPI_HANDLER, &Buffer[2], Bytes, MRAM_SPI_TIMEOUT);
		if (res != HAL_OK) return res;
		break;
	case MR25H256_WRITE:
		res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, &Buffer[0], Bytes + 2, MRAM_SPI_TIMEOUT);
		if (res != HAL_OK) return res;
		break;
	}
	BSP_MRAM_CS_INACTIVE();
}

int MRAM_Read(uint16_t address, uint8_t *data, int len)
{
	HAL_StatusTypeDef res;
	uint8_t ucCmd = MR25H256_READ;
	//uint16_t i;
	BSP_MRAM_CS_ACTIVE();
	res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, (uint8_t*)&ucCmd, 1, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return 0;
	res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, (uint8_t*)&address, 2, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return 0;
	res = HAL_SPI_Receive(&MRAM_SPI_HANDLER, (uint8_t*)data, (uint16_t)len, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return 0;
	BSP_MRAM_CS_INACTIVE();
	return len;
}

int MRAM_Write(uint16_t address, uint8_t *data, int len)
{
	HAL_StatusTypeDef res;
	uint8_t ucCmd = MR25H256_WRITE;
	BSP_MRAM_CS_ACTIVE();
	res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, &ucCmd, 1, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return 0;
	res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER, (uint8_t*)&address, 2, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return 0;
	res = HAL_SPI_Transmit(&MRAM_SPI_HANDLER,  data, len, MRAM_SPI_TIMEOUT);
	if (res != HAL_OK) return 0;
	BSP_MRAM_CS_INACTIVE();
	return len;
}

#ifdef USE_MRAM_COMMAND
#ifdef USE_CMDLINE
unsigned char MRAM_DBG(tPar *pPar, unsigned char ucParNum, unsigned char ucSwitch)
{
	unsigned char DataBuff[10], i;
	int cnt;
	switch(ucSwitch)
	{
		case 0 :
			DataBuff[0] = (pPar[1].x >> 8)&0xFF;
			DataBuff[1] = pPar[1].x & 0xFF;
			DataBuff[2] = pPar[2].x & 0xFF;
			MRAM_Transaction(pPar[0].x, DataBuff, 1);
			printf("\n\r");
			for (i=0; i<10; i++)
				printf("0x%x ",DataBuff[i]);
			//printf("\n\r");
			break;
		case 1 :
			//MR25H256_WRSR();
			MRAM_Transaction(MR25H256_RDSR, DataBuff, 1);
			printf("\n\rRDSR: 0x%x",DataBuff[0]);
			break;
		case 2 :
			//Memory Byte Read
			cnt = MRAM_Read(pPar[0].x, DataBuff, 1);
			printf("Read %d bytes: 0x%02x\r\n", cnt, DataBuff[0]);
			break;
		case 3 :
			//Memory Byte Write
			DataBuff[0] = pPar[1].x & 0xFF;
			cnt = MRAM_Write(pPar[0].x, DataBuff, 1);
			printf("Write %d bytes: 0x%02x\r\n", cnt, pPar[1].x);
			cnt = MRAM_Read(pPar[0].x, DataBuff, 1);
			printf("Read %d bytes: 0x%02x\r\n", cnt, DataBuff[0]);
			break;
	}//switch
return 0;
}
#endif
#endif








